package cn.fw.valhalla.domain.dto;

import cn.fw.common.validator.EnumValue;
import cn.fw.valhalla.domain.enums.PubStandType;
import lombok.Data;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.NotNull;
import java.util.Objects;

/**
 * 公共池站岗
 *
 * @author : kurisu
 * @version : 2.0
 * @className : PubStandDTO
 * @description : 公共池站岗
 * @date : 2023-04-13 19:19
 */
@Data
@ScriptAssert.List({
        @ScriptAssert(lang = "javascript", script = "_this.checkMktNo()", message = "活动信息不能为空"),
})
public class PubStandDTO {
    private Long id;
    /**
     * 站岗类型
     */
    @EnumValue(enumClass = PubStandType.class, message = "站岗类型不正确")
    @NotNull(message = "站岗类型不能为空")
    private Integer standType;
    /**
     * 站岗标识码「活动类型为活动编号」
     */
    private Long idCode;
    /**
     * 站岗范围
     */
    @EnumValue(enumClass = PubStandType.class, message = "站岗范围不正确")
    @NotNull(message = "站岗范围不能为空")
    private Integer standRang;

    public boolean checkMktNo() {
        if (PubStandType.ACTIVITY.getValue().equals(standType)) {
            return Objects.nonNull(idCode) && idCode > 0;
        }
        return true;
    }
}
