package cn.fw.valhalla.rpc.mkt.dto;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.customer.Customer;

import java.time.LocalDate;
import java.util.Date;
import java.util.Objects;
import java.util.Optional;

/**
 * 活动资格查询实体
 *
 * @author : kurisu
 * @version : 1.0
 * @className : QualificationDTO
 * @description : 活动资格查询实体
 * @date : 2023-03-25 17:08
 */
public class QualificationDTO {
    /**
     * 档案id不能为空
     */
    private Long archiveId;
    /**
     * 门店
     */
    private Long shopId;
    /**
     * 是否流失客户
     */
    private Boolean loseClue;
    /**
     * 是否续保线索
     */
    private Boolean renewalClue;
    /**
     * 车辆里程
     */
    private Integer mileage;
    /**
     * 车龄
     */
    private Integer age;
    /**
     * 客户等级
     */
    private Integer level;
    /**
     * 活动编号
     */
    private Long activityNo;

    public static QualificationDTO createWithCustomer(Customer customer) {
        QualificationDTO dto = new QualificationDTO();
        dto.setArchiveId(customer.getId());
        dto.setMileage(customer.getCurrentMileage());
        dto.setAge(0);
        if (Objects.nonNull(customer.getBuyDate()) || Objects.nonNull(customer.getRegDate())) {
            Date buyDate = Optional.ofNullable(customer.getBuyDate()).orElse(customer.getRegDate());
            dto.setAge(LocalDate.now().getYear() - DateUtil.date2LocalDate(buyDate).getYear() + 1);
        }
        dto.setLevel(customer.getCusLevel());
        return dto;
    }

    public Long getArchiveId() {
        return archiveId;
    }

    public void setArchiveId(final Long archiveId) {
        this.archiveId = archiveId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(final Long shopId) {
        this.shopId = shopId;
    }

    public Boolean getLoseClue() {
        return loseClue;
    }

    public void setLoseClue(final Boolean loseClue) {
        this.loseClue = loseClue;
    }

    public Boolean getRenewalClue() {
        return renewalClue;
    }

    public void setRenewalClue(final Boolean renewalClue) {
        this.renewalClue = renewalClue;
    }

    public Integer getMileage() {
        return mileage;
    }

    public void setMileage(final Integer mileage) {
        this.mileage = mileage;
    }

    public Integer getAge() {
        return age;
    }

    public void setAge(final Integer age) {
        this.age = age;
    }

    public Integer getLevel() {
        return level;
    }

    public void setLevel(final Integer level) {
        this.level = level;
    }

    public Long getActivityNo() {
        return activityNo;
    }

    public void setActivityNo(final Long activityNo) {
        this.activityNo = activityNo;
    }
}
