package cn.fw.valhalla.sdk.api;

import cn.fw.data.base.domain.common.Message;
import cn.fw.valhalla.sdk.param.FollowOriginalParam;
import cn.fw.valhalla.sdk.param.ShouldBeCompletedQuery;
import cn.fw.valhalla.sdk.result.ShouldBeCompletedResult;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowApiService
 * @description : 跟进相关api
 * @date: 2020-08-14 17:51
 */
@FeignClient(value = "fw-valhalla", path = "/api/valhalla/follow")
public interface FollowApiService {

    /**
     * 保存会产生跟进的业务数据
     *
     * @param originalParam
     * @return
     */
    @PostMapping("/origin/save")
    Message<Void> saveOriginal(@Valid @RequestBody FollowOriginalParam originalParam);

    /**
     * 移除业务数据
     *
     * @param type     {@link cn.fw.valhalla.sdk.enums.DataTypeEnum}
     * @param detailId
     * @return
     */
    @GetMapping("/origin/remove")
    Message<Void> removeData(@RequestParam("type") Integer type, @RequestParam("detailId") String detailId);


    /**
     * 查询续保跟进应成交台数
     *
     * @param query 查询参数
     * @return 分组人员
     */
    @GetMapping("/should/ins/completed")
    Message<List<ShouldBeCompletedResult>> queryInsShouldBeCompleted(@Validated @SpringQueryMap ShouldBeCompletedQuery query);

    /**
     * 查询续保跟进成交数据
     *
     * @param query 查询参数
     * @return 人员
     */
    @GetMapping("/ins/completed_arr")
    Message<List<ShouldBeCompletedResult>> queryInsCompletedGuys(@Validated @SpringQueryMap ShouldBeCompletedQuery query);
}