package cn.fw.valhalla.sdk.result;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.lang.Nullable;

import java.util.ArrayDeque;
import java.util.Date;

/**
 * 档案分配后的mq通知
 *
 * @author : kurisu
 * @version : 1.0
 * @className : CustomerDistributedMQ
 * @description : 档案分配后的mq通知
 * @date : 2023-03-15 15:39
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class CustomerDistributedMQ {
    public final static String TOPIC = "valhalla_customer_distribute";
    /**
     * 被分配的服务顾问id
     */
    private ArrayDeque<Long> staffIds;
    /**
     * 分配时间
     */
    private Date distributeTime;
    /**
     * 离职分配
     * <p>为false的时候为管理者主动分配</p>
     */
    private boolean resign;

    /**
     * 获取第一个staff
     *
     * @return
     */
    @Nullable
    public Long getStaffId() {
        return staffIds.pollFirst();
    }

    /**
     * 获取最后一个staff
     *
     * @return
     */
    @Nullable
    public Long getLastStaffId() {
        return staffIds.pollFirst();
    }

}
