package cn.fw.valhalla.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.vo.pool.PubPoolSimpleVO;
import cn.fw.valhalla.service.bus.pub.PubFollowBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 公共站岗池
 *
 * @author : kurisu
 * @version : 2.0
 * @className : PubClueController
 * @description : 公共站岗池
 * @date : 2023-04-12 16:42
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/pub/pool")
public class PubClueController {

    private final PubFollowBizService pubFollowBizService;

    @Autowired
    public PubClueController(final PubFollowBizService pubFollowBizService) {
        this.pubFollowBizService = pubFollowBizService;
    }

    @GetMapping("/log")
    @ControllerMethod("查看分配日志")
    public Message<List<PubPoolSimpleVO>> info(@CurrentUser final LoginAuthBean user) {
        return success(pubFollowBizService.log(user.getUserId(), user.getGroupId()));
    }
}
