package cn.fw.valhalla.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.PubStandDTO;
import cn.fw.valhalla.domain.vo.pool.PubStandVO;
import cn.fw.valhalla.service.bus.pub.PubStandBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 公共线索站岗
 *
 * @author : kurisu
 * @version : 2.0
 * @className : PubStandController
 * @description : 公共线索站岗
 * @date : 2023-04-12 14:41
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/pub/stand")
public class PubStandController {
    private final PubStandBizService pubStandBizService;

    @Autowired
    public PubStandController(final PubStandBizService pubStandBizService) {
        this.pubStandBizService = pubStandBizService;
    }

    @GetMapping("/info")
    @ControllerMethod("人员站岗信息")
    public Message<PubStandVO> info(@CurrentUser final LoginAuthBean user) {
        return success(pubStandBizService.standInfo(user));
    }

    @PostMapping("/update")
    @ControllerMethod("人员站岗信息")
    public Message<Void> update(@CurrentUser final LoginAuthBean user, @Validated @RequestBody PubStandDTO standDTO) {
        pubStandBizService.updateInfo(user, standDTO);
        return success();
    }

    @PutMapping("/join")
    @ControllerMethod("加入站岗队列")
    public Message<Void> join(@CurrentUser final LoginAuthBean user) {
        pubStandBizService.join(user.getUserId(), true);
        return success();
    }

    @PutMapping("/exit")
    @ControllerMethod("退出站岗队列")
    public Message<Void> exit(@CurrentUser final LoginAuthBean user) {
        pubStandBizService.exitStand(user.getUserId(), user.getGroupId());
        return success();
    }
}
