package cn.fw.valhalla.controller.erp;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.db.query.Page;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.query.MemberVehicleQueryVO;
import cn.fw.valhalla.domain.vo.IdVO;
import cn.fw.valhalla.domain.vo.customer.MemberVehicleVO;
import cn.fw.valhalla.service.bus.cust.MemberVehicleBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import static cn.fw.data.base.domain.common.Message.success;

/**
 * 会员认证车辆
 * <p>
 * create at 2022-06-06
 *
 * @author xiatian
 */
@Slf4j
@Validated
@RestController
@Authorization(AuthType.ERP)
@RequiredArgsConstructor
@RequestMapping("/erp/member/vehicle")
public class MemberVehicleController {

    private final MemberVehicleBizService memberVehicleBizService;

    @IgnoreAuth
    @GetMapping("/page")
    @ControllerMethod("分页查询会员认证车辆")
    Message<Page<MemberVehicleVO>> page(MemberVehicleQueryVO vo) {
        return success(memberVehicleBizService.page(vo));
    }

    @IgnoreAuth
    @PostMapping("/delete")
    @ControllerMethod("删除会员认证车辆")
    Message<Void> delete(@Valid @RequestBody IdVO vo) {
        memberVehicleBizService.deleteVehicle(vo);
        return success();
    }


}
