package cn.fw.valhalla.component.consumer

import cn.fw.attendance.sdk.api.enums.ClockInTypeEnum
import cn.fw.attendance.sdk.api.mq.ClockInInfo
import cn.fw.valhalla.service.bus.pub.PubStandBizService
import lombok.extern.slf4j.Slf4j
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener
import org.apache.rocketmq.spring.core.RocketMQListener
import org.springframework.stereotype.Component

/**
 * 上班打卡消费类
 *
 * @author : kurisu
 * @version : 2.1
 * @className : CheckInConsumer
 * @description : 上班打卡消费类
 * @date : 2023-04-07 15:12
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = ClockInInfo.TOPIC, consumerGroup = "\${spring.application.name}_pub_stand")
class CheckInConsumer(private val pubStandBizService: PubStandBizService) : RocketMQListener<ClockInInfo> {
    override fun onMessage(message: ClockInInfo?) {
        message?.let {
            if (ClockInTypeEnum.START_WORK.value.equals(it.clockInType)) {
                pubStandBizService.join(it.userId, false)
            }
            if (ClockInTypeEnum.END_WORK.value.equals(it.clockInType)) {
                pubStandBizService.endWorkExitStand(it.userId)
            }
        }
    }
}