package cn.fw.valhalla.component.consumer;

import cn.fw.shirasawa.sdk.mq.FollowApproveDTO
import cn.fw.valhalla.domain.enums.FollowTypeEnum.PL
import cn.fw.valhalla.domain.enums.FollowTypeEnum.ofValue
import cn.fw.valhalla.domain.enums.PublicPoolTypeEnum
import cn.fw.valhalla.service.bus.cust.CustomerBizService
import cn.fw.valhalla.service.data.ClueTaskService
import cn.fw.valhalla.service.data.PubCluePoolService
import com.alibaba.fastjson.JSONObject
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener
import org.apache.rocketmq.spring.core.RocketMQListener
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Component

/**
 * @author : kurisu
 * Date: 2020/9/16
 * Time: 10:57
 * Description:
 */
@Component
@RocketMQMessageListener(
    topic = FollowApproveDTO.FOLLOW_APPROVE_TOPIC,
    consumerGroup = "\${spring.application.name}_follow_flow_result"
)
class FollowFlowConsumer(
    private val customerBizService: CustomerBizService,
    private val clueTaskService: ClueTaskService,
    private val pubCluePoolService: PubCluePoolService
) : RocketMQListener<FollowApproveDTO> {

    private val log: Logger = LoggerFactory.getLogger(this.javaClass)

    override fun onMessage(dto: FollowApproveDTO?) {
        log.info("跟进审批通过MQ消息回调：{}", JSONObject.toJSONString(dto));
        dto?.let { data ->
            clueTaskService.queryOngoingTaskByClueId(data.detailId.toLong(), ofValue(data.type))?.apply {
                when (this.type) {
                    PL -> {
                        pubCluePoolService.getById(this.clueId)?.let {
                            customerBizService.abandon(it.vin, it.groupId, PublicPoolTypeEnum.AD)
                        }
                    }

                    else -> {
                        customerBizService.abandon(this)
                    }
                }
            }
        }
    }
}
