package cn.fw.valhalla.component.consumer;

import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.sdk.mq.FollowResultDTO;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.bus.pub.PubStandBizService;
import cn.fw.valhalla.service.data.FollowClueService;
import cn.hutool.core.collection.ListUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.Objects;
import java.util.Set;

/**
 * @author : kurisu
 * Date: 2020/9/16
 * Time: 10:57
 * Description:
 */
@Slf4j
@Component
@RocketMQMessageListener(topic = FollowResultDTO.FOLLOW_TOPIC, consumerGroup = "${spring.application.name}_follow_todo_result")
public class FollowResultConsumer implements RocketMQListener<FollowResultDTO> {

    private final FollowBizService followBizService;
    private final FollowClueService followClueService;
    private final PubStandBizService pubStandBizService;


    @Autowired
    public FollowResultConsumer(final FollowBizService followBizService,
                                final FollowClueService followClueService,
                                final PubStandBizService pubStandBizService) {
        this.followBizService = followBizService;
        this.followClueService = followClueService;
        this.pubStandBizService = pubStandBizService;
    }

    @Override
    public void onMessage(FollowResultDTO dto) {
        log.info("处理跟进结果MQ消息：{}", JSONObject.toJSONString(dto));
        try {
            LinkedList<Integer> linkedList = ListUtil.toLinkedList(
                    DataTypeEnum.AC.getValue(),
                    DataTypeEnum.FM.getValue(),
                    DataTypeEnum.RM.getValue(),
                    DataTypeEnum.IR.getValue(),
                    DataTypeEnum.PL.getValue()
            );
            Set<Integer> validSet = new HashSet<>(linkedList);
            if (!validSet.contains(dto.getType())) {
                return;
            }
            pubStandBizService.join(dto.getUserId(), false);
            Long val = Long.valueOf(dto.getDetailId());
            if (DataTypeEnum.PL.getValue().equals(dto.getType())) {
                FollowClue clue = new FollowClue();
                clue.setId(val);
                clue.setClueType(FollowTypeEnum.PL);
                followBizService.afterFollowComplete(clue, dto.isOverdue());
                return;
            }
            FollowClue clue = followClueService.getById(val);
            if (Objects.isNull(clue)) {
                return;
            }
            followBizService.afterFollowComplete(clue, dto.isOverdue());
        } catch (Exception e) {
            log.error("处理跟进结果MQ异常", e);
            throw e;
        }
    }
}
