package cn.fw.valhalla.component.producer

import cn.fw.valhalla.sdk.result.CustomerDistributedMQ
import org.apache.rocketmq.spring.core.RocketMQTemplate
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Component
import org.springframework.web.bind.annotation.RequestMapping
import java.util.*

/**
 * 保有客分配后的mq生产者
 *
 * @author : kurisu
 * @version : 1.0
 * @className : CustAfterDistProducer
 * @description : 保有客分配后的mq生产者
 * @date : 2023-03-15 15:37
 */
@Component
class CustAfterDistProducer(private val rocketMQTemplate: RocketMQTemplate) {
    private val log: Logger = LoggerFactory.getLogger(this.javaClass)

    @RequestMapping(value = ["send"])
    fun send(resign: Boolean, staffIds: List<Long>?) {
        log.info("发送保有客分配结束mq消息。staffIds:{}", staffIds)
        try {
            when {
                staffIds.isNullOrEmpty() -> return
                else -> {
                    val distributedMQ = CustomerDistributedMQ(ArrayDeque(staffIds), Date(), resign)
                    rocketMQTemplate.syncSend(CustomerDistributedMQ.TOPIC + ":*", distributedMQ)
                }
            }
        } catch (e: Exception) {
            log.info("发送保有客分配结束mq消息失败！", e)
        }
    }
}
