package cn.fw.valhalla.service.bus.follow.strategy.impl;

import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.OriginalData;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.shirasawa.dto.FollowInitDTO;
import cn.fw.valhalla.sdk.enums.DataTypeEnum;
import cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : FMFollowStrategy
 * @description : 首保策略
 * @date: 2020-08-17 10:46
 */
@Slf4j
@Component
public class FMFollowStrategy extends AbstractFollowStrategy {
    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.FM;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean origin2task(OriginalData originalData) {
        Customer customer = null;
        if (StringUtils.isValid(originalData.getFrameNo())) {
            customer = customerService.queryByFrameNo(originalData.getFrameNo(), originalData.getGroupId());
        }
        if (Objects.isNull(customer)) {
            customer = customerService.queryById(originalData.getCustomerId());
        }
        BV.notNull(customer, () -> "档案不存在");
        if (DataTypeEnum.FM.equals(originalData.getType())) {
            FollowClue followClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                            .eq(FollowClue::getVin, customer.getFrameNo())
                            .eq(FollowClue::getClueType, getFollowType())
                    , Boolean.FALSE);
            if (Objects.isNull(followClue)) {
                return true;
            }
            if (ClueStatusEnum.WAITING.equals(followClue.getClueState())) {
                followClueService.removeById(followClue.getId());
            } else if (ClueStatusEnum.ONGOING.equals(followClue.getClueState())) {
                finishClue(originalData, followClue);
            }
            return true;
        }

        FollowClue followClue = followClueService.getOne(Wrappers.<FollowClue>lambdaQuery()
                        .eq(FollowClue::getVin, customer.getFrameNo())
                        .eq(FollowClue::getClueType, getFollowType())
                        .eq(FollowClue::getClueState, ClueStatusEnum.WAITING)
                , Boolean.FALSE);
        if (DataTypeEnum.OD.equals(originalData.getType()) && Objects.nonNull(followClue)) {
            followClueService.removeById(followClue.getId());
        }
        FollowClue clue = createClueInfo(originalData, getFollowType(), customer);
        followClueService.save(clue);
        createFirstNotice(clue, customer.getId(), customer.getBrandId());
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void startClue(FollowClue followClue) {
        if (!ClueStatusEnum.WAITING.equals(followClue.getClueState())) {
            return;
        }
        Customer customer = customerService.queryByFrameNo(followClue.getVin(), followClue.getGroupId());
        if (Objects.isNull(customer) || Objects.isNull(customer.getAdviserId())) {
            followClueService.removeById(followClue.getId());
            return;
        }
        followClue.setPlateNo(customer.getPlateNo());
        final ClueTask clueTask = super.createNewTask(followClue);
        List<PostUserDTO> userByRole = userService.getUserByRole(clueTask.getFollowShop(), RoleCode.FWGW);
        BV.isNotEmpty(userByRole, () -> "该门店没有【服务接待】人员");
        fillTaskUser(clueTask, userByRole);
        followClue.setClueState(ClueStatusEnum.ONGOING);
        clueTaskService.save(clueTask);
        final FollowInitDTO followInitDTO = creteFollowInitDTO(followClue, clueTask);
        shirasawaRpcService.createFollowData(followInitDTO);
        followClueService.updateById(followClue);
    }
}
